<!DOCTYPE html>
<html lang="en"><head>
  <meta charset="UTF-8">
  <meta name="generator" content="{{  generator }}">
  <title>{{ title }}</title>
  </head><body style="font-family:{{ options.fontname }};background-color:{{ options.bgcolor }}">
<h1>{{ title }}</h1>
<h2>Diagram</h2>

<div id="description">
  {{ description }}
</div>

<div id="diagram">
  {{ diagram }}
</div>

<div id="notes">
  {{ notes }}
</div>

<h2>Bill of Materials</h2>

  {{ bom }}

</body></html>
