<!-- Component definition Table !-->
<table border="0" cellborder="0" cellspacing="0" cellpadding="0" bgcolor="#FFFFFF">
  <!-- Header !-->
  <tr>
    <td>
      <table border="0" cellborder="1" cellspacing="0" cellpadding="3" bgcolor="#FFFFFF">
        <tr>
          <td colspan="2"> <b> {{ component.designator }} </b> </td>
          {% block header_extra_col %} {% endblock %}
          {# Doesn't work with cable components, because they can have a list of bom entries
          {% if component.bom_entry.id %}
          <td> BOM: {{ component.bom_entry.id }} </td>
          {% endif %}
          #}
        </tr>
      </table>
    </td>
  </tr>
  <!-- Part Number !-->
  {% if component.partnumbers %}
      {% if component.partnumbers.is_list %}
          {% set item = component.partnumbers.keep_only_shared().as_list() %}
      {% else %}
          {% set item = component.partnumbers.as_list() %}
      {% endif %}
      {% if item %}
      <tr>
        <td>
          <table border="0" cellborder="1" cellpadding="3" cellspacing="0">
            <tr>

              {% for pn in item %}
                  <td> {{ pn }} </td>
              {% endfor %}
            </tr>
          </table>
        </td>
      </tr>
      {% endif %}
  {% endif %}
  <!-- Info !-->
  <tr>
    <td>
      <table border="0" cellborder="1" cellpadding="3" cellspacing="0">
        <tr>
          {% block info_columns %} {% endblock %}
        </tr>
      </table>
    </td>
  </tr>
  <!-- Ports !-->
  <tr>
    <td>
      {% block ports %} {% endblock %}
    </td>
  </tr>
  <!-- Image !-->
  {% if component.image %}
  {% set image=component.image %}
  <tr >
    <td>
      <table border="0" cellborder="1" cellspacing="0" cellpadding="3" bgcolor="#FFFFFF">
        <tr>
          <td>
              {% include "images.html" %}
          </td>
        </tr>
      </table>
    </td>
  </tr>
  {% endif %}
  <!-- Additional Components !-->
  {% if component.additional_components %}
  {% set additional_components=component.additional_components %}
  <tr>
    <td>
        {% include "additional_components.html" %}
    </td>
  </tr>
  {% endif %}
  <!-- Notes !-->
  {% if component.notes and component.notes.raw %}
  <tr>
    <td>
      <table border="0" cellborder="1" cellpadding="3" cellspacing="0">
        <tr>
          <td> {{ component.notes.clean }} </td>
        </tr>
      </table>
    </td>
  </tr>
  {% endif %}
</table>
